/* ******************************* */
/*    Naka. BGM Player ver.0.10    */
/*                                 */
/*                 NKBGM Header    */
/*                      NKBGM.H    */
/* ******************************* */

#ifndef _NKBGM_H
#define _NKBGM_H

#include <windows.h>

/* ********************************************************* Definition **** */

#define DEFAULT_TEMPO		500000	// Tempo = 120

#define META_TEMPO			0x51

#define CHUNK_HEADER		"MThd"
#define CHUNK_TRACK			"MTrk"

#define MTRK_EVENT_MIDI		0x00
#define MTRK_EVENT_SYSEX0	0xF0
#define MTRK_EVENT_SYSEX7	0xF7
#define MTRK_EVENT_META		0xFF

#define BGM_NODATA			0x0000
#define BGM_READY			0x0001
#define BGM_PLAY			0x0002
#define BGM_STOP			0x0008
#define BGM_PAUSE			0x0010
#define BGM_FADEOUT			0x0800

#define BGM_REPEAT_ON		0x8000
#define BGM_REPEAT_OFF		0x0000

#define TRACK_STOP			0xffffffff

/* *************************************************** Define structure **** */

typedef union FMTCHG_tag{
	BYTE	b[4];
	WORD	w[2];
	DWORD	dw;
} FMTCHG;

typedef struct VARLEN_tag{
	FMTCHG	varlen;
	DWORD	value;
	UINT	size;					// Size of variable length
} VARLEN;

typedef struct LONGMSG_tag{
	HGLOBAL		hMidiHdr;
	MIDIHDR		*pMidiHdr;
	HGLOBAL		hLongData;
} LONGMSG;

typedef struct EVENTINFO_tag{
	DWORD	dwCount;				// Event Count
	DWORD	dwTime;					// Event Time
	UINT	uEvent;					// Kind of Event
	UINT	uEntry;					// Entry of pData
	UINT	uSize;					// Size of play data
} EVENTINFO;

typedef struct TRACKINFO_tag{
	UINT		uSize;
	UINT		uEvents;
	UINT		uEvent;
	HGLOBAL		hData,hEvent;
	DWORD		dwTime;				// Length of Track
	BYTE		*pData;
	EVENTINFO	*pEvent;
} TRACKINFO;

typedef struct TEMPOMAP_tag{
	DWORD		dwCount;
	DWORD		dwTempo;
	DWORD		dwTime;
} TEMPOMAP;

typedef struct SONGINFO_tag{
	UINT		uFormat;
	UINT		uTracks;
	UINT		uDivision;
	HGLOBAL		hTrack;
	TRACKINFO	*pTrack;
	DWORD		dwTime;					// Length of Song
	UINT		uEndTracks;
} SONGINFO;

typedef struct MIDIINFO_tag{
	HMIDIOUT	hMidiOut;				// Handle of MIDI Output Device
	UINT		uDeviceID;				// MIDI Output Device ID
	UINT		uMidiStatus;			// MIDI Error Status
	UINT		uBgmStatus;				// BGM Status
	UINT		uRepeat;				// Repeat Flag
	UINT		uPeriod;				// Timer Period
	UINT		uTimerID;				// Timer ID
	HGLOBAL		hSong;					// Handle of SONGINFO Memory
	SONGINFO	*pSong;					// Pointer of SONFINFO Memory
	DWORD		dwTime;					// Playing Time
	LONGMSG		LongMsg;
} MIDIINFO;

#pragma pack(2)
typedef struct SMFBLOCK_tag{
	char	chunk[4];
	DWORD	length;
} SMFBLOCK;

typedef struct SMFHEAD_tag{
	SMFBLOCK	block;
	WORD		format;
	WORD		ntrks;
	WORD		division;
} SMFHEAD;
#pragma pack()

/* ********************************************************** Prototype **** */

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _DLL_EXPORT_			// Export DLL
#define CLASS_DECLSPEC    __declspec(dllexport)
#else						// Import DLL
#define CLASS_DECLSPEC    __declspec(dllimport)
#endif

/* ------------------------------------------------------ DLL Fucntions ---- */

CLASS_DECLSPEC UINT uMidiStatus;	// MIDI Status

CLASS_DECLSPEC HGLOBAL	WINAPI bgmOpenMidiDev(UINT, UINT);
CLASS_DECLSPEC UINT		WINAPI bgmCloseMidiDev(HGLOBAL);

CLASS_DECLSPEC UINT 	WINAPI bgmPlay(HGLOBAL, HGLOBAL);
CLASS_DECLSPEC UINT 	WINAPI bgmStop(HGLOBAL);
CLASS_DECLSPEC UINT 	WINAPI bgmPause(HGLOBAL);

CLASS_DECLSPEC UINT 	WINAPI bgmSetRepeat(HGLOBAL, UINT);
CLASS_DECLSPEC UINT		WINAPI bgmGetRepeat(HGLOBAL);

CLASS_DECLSPEC void 	WINAPI bgmFadeout(HGLOBAL);

CLASS_DECLSPEC HGLOBAL	WINAPI bgmGetSongFile(char *);
CLASS_DECLSPEC HGLOBAL 	WINAPI bgmGetSongRsc(HINSTANCE, char *);
CLASS_DECLSPEC HGLOBAL 	WINAPI bgmDeleteSong(HGLOBAL);
CLASS_DECLSPEC DWORD	WINAPI bgmGetSongLength(HGLOBAL);

CLASS_DECLSPEC UINT		WINAPI bgmBgmStatus(HGLOBAL);
CLASS_DECLSPEC UINT		WINAPI bgmMidiStatus(HGLOBAL);
CLASS_DECLSPEC UINT		WINAPI bgmGetMidiErrorText(char *, UINT);

#ifdef __cplusplus
}
#endif

#endif		// _NKBGM_H
